# coding: utf-8
"""Implementation of Arlo utils."""
import time
import requests


def pretty_timestamp(timestamp, date_format='%a-%m_%d_%y:%H:%M:%S'):
    """Huminize timestamp."""
    return time.strftime(date_format,
                         time.localtime(int(str(timestamp)[:10])))


def http_get(url, filename=None):
    """Download HTTP data."""
    ret = requests.get(url)
    if ret.status_code != 200:
        return False

    if filename is None:
        return ret.content
    else:
        with open(filename, 'wb') as data:
            data.write(ret.content)
    return True


def http_stream(url, chunk=4096):
    """Generate stream for a given record video.

    :param chunk: chunk bytes to read per time
    :returns generator object
    """
    ret = requests.get(url, stream=True)
    ret.raise_for_status()
    for data in ret.iter_content(chunk):
        yield data


# vim:sw=4:ts=4:et:



"""Constants used by Python Arlo."""

# API Endpoints
API_URL = "https://arlo.netgear.com/hmsweb"

DEVICE_SUPPORT_ENDPOINT = API_URL + "/devicesupport/v2"
SUBSCRIBE_ENDPOINT = API_URL + "/client/subscribe"
UNSUBSCRIBE_ENDPOINT = API_URL + "/client/unsubscribe"
BILLING_ENDPOINT = API_URL + "/users/serviceLevel/v2"
DEVICES_ENDPOINT = API_URL + "/users/devices"
FRIENDS_ENDPOINT = API_URL + "/users/friends"
LIBRARY_ENDPOINT = API_URL + "/users/library"
LOGIN_ENDPOINT = API_URL + "/login/v2"
LOGOUT_ENDPOINT = API_URL + "/logout"
NOTIFY_ENDPOINT = API_URL + "/users/devices/notify/{0}"
PROFILE_ENDPOINT = API_URL + "/users/profile"
RESET_ENDPOINT = LIBRARY_ENDPOINT + "/reset"
RESET_CAM_ENDPOINT = RESET_ENDPOINT + "/?uniqueId={0}"
STREAM_ENDPOINT = API_URL + "/users/devices/startStream"

# number of days to preload video
PRELOAD_DAYS = 30

# define action modes
ACTION_MODES = {
    'armed': 'mode1',
    'disarmed': 'mode0',
    'custom': 'mode2',
    'schedule': 'true',
}

# define resources
RESOURCES = {
    'base_station': 'base_station',
    'modes': 'modes',
    'schedule': 'schedule',
    'rules': 'rules',
    'cameras': 'cameras',
}

# define body used when executing an action
ACTION_BODY = {
    'action': None,
    'from': None,
    'properties': None,
    'publishResponse': None,
    'resource': None,
    'to': None
}

# define body used for live_streaming
STREAMING_BODY = {
    'action': 'set',
    'from': None,
    'properties': {'activityState': 'startPositionStream'},
    'publishResponse': 'true',
    'resource': None,
    'to': None,
    'transId': "",
}

# vim:sw=4:ts=4:et:



# coding: utf-8
"""Implementation of Arlo Media object."""
import logging
from datetime import datetime
from datetime import timedelta
from pyarlo.const import LIBRARY_ENDPOINT, PRELOAD_DAYS
from pyarlo.utils import http_get, http_stream, pretty_timestamp

_LOGGER = logging.getLogger(__name__)


class ArloMediaLibrary(object):
    """Arlo Library Media module implementation."""

    def __init__(self, arlo_session, preload=True, days=PRELOAD_DAYS):
        """Initialiaze Arlo Media Library object.

        :param arlo_session: PyArlo shared session
        :param preload: Boolean to pre-load video library.
        :param days: If preload, number of days to lookup.

        :returns ArloMediaLibrary object
        """
        self._session = arlo_session

        if preload and days:
            self.videos = self.load(days)
        else:
            self.videos = []

    def __repr__(self):
        """Representation string of object."""
        return "<{0}: {1}>".format(self.__class__.__name__,
                                   self._session.userid)

    def load(self, days=PRELOAD_DAYS, only_cameras=None,
             date_from=None, date_to=None, limit=None):
        """Load  Arlo videos from the given criteria

        :param days: number of days to retrieve
        :param only_cameras: retrieve only <ArloCamera> on that list
        :param date_from: refine from initial date
        :param date_to: refine final date
        :param limit: define number of objects to return
        """
        videos = []
        url = LIBRARY_ENDPOINT
        if not (date_from and date_to):
            now = datetime.today()
            date_from = (now - timedelta(days=days)).strftime('%Y%m%d')
            date_to = now.strftime('%Y%m%d')

        params = {'dateFrom': date_from, 'dateTo': date_to}
        data = self._session.query(url,
                                   method='POST',
                                   extra_params=params).get('data')

        # get all cameras to append to create ArloVideo object
        all_cameras = self._session.cameras

        for video in data:
            # pylint: disable=cell-var-from-loop
            srccam = \
                list(filter(
                    lambda cam: cam.device_id == video.get('deviceId'),
                    all_cameras)
                    )[0]

            # make sure only_cameras is a list
            if only_cameras and \
               not isinstance(only_cameras, list):
                only_cameras = list(only_cameras)

            # filter by camera only
            if only_cameras:
                if list(filter(lambda cam: cam.device_id == srccam.device_id,
                               list(only_cameras))):
                    videos.append(ArloVideo(video, srccam, self._session))
            else:
                videos.append(ArloVideo(video, srccam, self._session))

        if limit:
            return videos[:limit]

        return videos


class ArloVideo(object):
    """Object for Arlo Video file."""

    def __init__(self, attrs, camera, arlo_session):
        """Initialiaze Arlo Video object.

        :param attrs: Video attributes
        :param camera: Arlo camera which recorded the video
        :param arlo_session: Arlo shared session
        """
        self._attrs = attrs
        self._camera = camera
        self._session = arlo_session

    def __repr__(self):
        """Representation string of object."""
        return "<{0}: {1}>".format(self.__class__.__name__, self._name)

    @property
    def _name(self):
        """Define object name."""
        return "{0} {1} {2}".format(
            self._camera.name,
            pretty_timestamp(self.created_at),
            self._attrs.get('mediaDuration'))

    # pylint: disable=invalid-name
    @property
    def id(self):
        """Return object id."""
        return self._attrs.get('name')

    @property
    def created_at(self):
        """Return timestamp."""
        return self._attrs.get('localCreatedDate')

    def created_at_pretty(self, date_format=None):
        """Return pretty timestamp."""
        if date_format:
            return pretty_timestamp(self.created_at, date_format=date_format)
        return pretty_timestamp(self.created_at)

    @property
    def content_type(self):
        """Return content_type."""
        return self._attrs.get('contentType')

    @property
    def camera(self):
        """Return camera object that recorded video."""
        return self._camera

    @property
    def media_duration_seconds(self):
        """Return media duration in seconds."""
        return self._attrs.get('mediaDurationSecond')

    @property
    def triggered_by(self):
        """Return the reason why video was recorded."""
        return self._attrs.get('reason')

    @property
    def thumbnail_url(self):
        """Return thumbnail url."""
        return self._attrs.get('presignedThumbnailUrl')

    @property
    def video_url(self):
        """Return video content url."""
        return self._attrs.get('presignedContentUrl')

    def download_thumbnail(self, filename=None):
        """Download JPEG thumbnail.

        :param filename: File to save thumbnail. Default: stdout
        """
        return http_get(self.thumbnail_url, filename)

    def download_video(self, filename=None):
        """Download video content.

        :param filename: File to save video. Default: stdout
        """
        return http_get(self.video_url, filename)

    @property
    def stream_video(self):
        """Stream video."""
        return http_stream(self.video_url)

# vim:sw=4:ts=4:et:



# coding: utf-8
"""Generic Python Class file for Netgear Arlo Base Station module."""
import json
import threading
import time
import logging
import sseclient
from pyarlo.const import (
    ACTION_BODY, SUBSCRIBE_ENDPOINT, UNSUBSCRIBE_ENDPOINT,
    ACTION_MODES, NOTIFY_ENDPOINT, RESOURCES)
_LOGGER = logging.getLogger(__name__)


class ArloBaseStation(object):
    """Arlo Base Station module implementation."""

    def __init__(self, name, attrs, session_token, arlo_session):
        """Initialize Arlo Base Station object.

        :param name: Base Station name
        :param attrs: Attributes
        :param session_token: Session token passed by camera class
        :param arlo_session: PyArlo shared session
        """
        self.name = name
        self._attrs = attrs
        self._session = arlo_session
        self._session_token = session_token
        self.__sseclient = None
        self.__subscribed = False
        self.__events = []

    def __repr__(self):
        """Representation string of object."""
        return "<{0}: {1}>".format(self.__class__.__name__, self.name)

    def thread_function(self):
        """Thread function."""
        url = SUBSCRIBE_ENDPOINT + "?token=" + self._session_token
        data = self._session.query(url, method='GET', raw=True, stream=True)
        self.__sseclient = sseclient.SSEClient(data)
        if self.__sseclient:
            self.__subscribed = True
        for event in (self.__sseclient).events():
            if not self.__subscribed:
                break
            data = json.loads(event.data)
            if data.get('status') == "connected":
                _LOGGER.debug("Successfully subscribed this base station")
            elif data.get('action'):
                action = data.get('action')
                resource = data.get('resource')
                if action == "logout":
                    _LOGGER.debug("Logged out by some other entity")
                    self.__subscribed = False
                    break
                elif action == "is" and "subscriptions/" not in resource:
                    self.__events.append(data)

    def _get_event_stream(self):
        """Spawn a thread and monitor the Arlo Event Stream."""
        event_thread = threading.Thread(target=self.thread_function)
        event_thread.start()

    def _subscribe_myself(self):
        """Subscribe this base station for all events."""
        return self.__run_action(
            method='SET',
            resource='subscribe',
            mode=None,
            publish_response=False)

    def _unsubscribe_myself(self):
        """Unsubscribe this base station for all events."""
        url = UNSUBSCRIBE_ENDPOINT
        return self._session.query(url, method='GET', raw=True, stream=False)

    def _close_event_stream(self):
        """Stop the Event stream thread."""
        self.__subscribed = False

    def publish_and_get_event(self, resource):
        """Publish and get the event from base station."""
        self._get_event_stream()
        self._subscribe_myself()

        this_event = ''
        status = self.__run_action(
            method='GET',
            resource=resource,
            mode=None,
            publish_response=False)
        if status == 'success':
            for i in range(0, 5):
                _LOGGER.debug("Trying instance " + str(i))
                time.sleep(1)
                for event in self.__events:
                    if event['resource'] == resource:
                        this_event = event
                        self.__events.remove(event)
                        break
                if this_event:
                    break

        self._unsubscribe_myself()
        self._close_event_stream()

        if this_event:
            return this_event

        return None

    def __run_action(
            self,
            method='GET',
            resource=None,
            mode=None,
            publish_response=None):
        """Run action.

        :param method: Specify the method GET, POST or PUT. Default is GET.
        :param resource: Specify one of the resources to fetch from arlo.
        :param mode: Specify the mode to set, else None for GET operations
        :param publish_response: Set to True for SETs. Default False
        """
        url = NOTIFY_ENDPOINT.format(self.device_id)

        body = ACTION_BODY

        if resource:
            body['resource'] = resource

        if method == 'GET':
            body['action'] = "get"
            body['properties'] = None
        elif method == 'SET':
            body['action'] = "set"
            if resource == 'schedule':
                body['properties'] = {'active': 'true'}
            elif resource == 'subscribe':
                body['resource'] = "subscriptions/" + \
                        "{0}_web".format(self.user_id)
                dev = []
                dev.append(self.device_id)
                body['properties'] = {'devices': dev}
            elif resource == 'modes':
                body['properties'] = {'active': ACTION_MODES.get(mode)}
        else:
            _LOGGER.info("Invalid method requested")
            return None

        if not publish_response:
            body['publishResponse'] = 'false'
        else:
            body['publishResponse'] = 'true'

        body['from'] = "{0}_web".format(self.user_id)
        body['to'] = self.device_id
        body['transId'] = "web!e6d1b969.8aa4b!1498165992111"

        _LOGGER.info("Action body: %s", body)

        ret = \
            self._session.query(url, method='POST', extra_params=body,
                                extra_headers={"xCloudId": self.xcloud_id})
        if ret.get('success'):
            return 'success'

        return None

    # pylint: disable=invalid-name
    @property
    def device_id(self):
        """Return device_id."""
        return self._attrs.get('deviceId')

    @property
    def device_type(self):
        """Return device_type."""
        return self._attrs.get('deviceType')

    @property
    def model_id(self):
        """Return model_id."""
        return self._attrs.get('modelId')

    @property
    def hw_version(self):
        """Return hardware version."""
        return self._attrs.get('properties').get('hwVersion')

    @property
    def timezone(self):
        """Return timezone."""
        return self._attrs.get('properties').get('olsonTimeZone')

    @property
    def unique_id(self):
        """Return unique_id."""
        return self._attrs.get('uniqueId')

    @property
    def serial_number(self):
        """Return serial number."""
        return self._attrs.get('properties').get('serialNumber')

    @property
    def user_id(self):
        """Return userID."""
        return self._attrs.get('userId')

    @property
    def user_role(self):
        """Return userRole."""
        return self._attrs.get('userRole')

    @property
    def xcloud_id(self):
        """Return X-Cloud-ID attribute."""
        return self._attrs.get('xCloudId')

    @property
    def available_modes(self):
        """Return list of available modes."""
        return list(ACTION_MODES.keys())

    @property
    def available_resources(self):
        """Return list of available resources."""
        return list(RESOURCES.keys())

    @property
    def mode(self):
        """Return current mode."""
        resource = "modes"
        mode_event = self.publish_and_get_event(resource)
        if mode_event:
            active_mode = mode_event['properties']['active']
            modes = mode_event['properties']['modes']
            for mode in modes:
                if mode['id'] == active_mode:
                    return mode['type']

        return None

    @property
    def get_camera_properties(self):
        """Return camera properties."""
        resource = "cameras"
        resource_event = self.publish_and_get_event(resource)
        if resource_event:
            return resource_event['properties']

        return None

    @property
    def get_camera_battery_level(self):
        """Return a list of battery levels of all cameras."""
        battery_levels = {}
        resource = "cameras"
        resource_event = self.publish_and_get_event(resource)
        if resource_event:
            cameras = resource_event['properties']
            for camera in cameras:
                battery_levels[camera['serialNumber']] = camera['batteryLevel']
            return battery_levels

        return None

    @property
    def get_basestation_properties(self):
        """Return the base station info."""
        resource = "basestation"
        basestn_event = self.publish_and_get_event(resource)
        if basestn_event:
            return basestn_event['properties']

        return None

    @property
    def get_camera_rules(self):
        """Return the camera rules."""
        resource = "rules"
        rules_event = self.publish_and_get_event(resource)
        if rules_event:
            return rules_event['properties']

        return None

    @property
    def get_camera_schedule(self):
        """Return the schedule set for cameras."""
        resource = "schedule"
        schedule_event = self.publish_and_get_event(resource)
        if schedule_event:
            return schedule_event['properties']

        return None

    @property
    def is_motion_detection_enabled(self):
        """Return Boolean if motion is enabled."""
        return bool(self.mode == "armed" or self.get_camera_schedule['active'])

    @property
    def subscribe(self):
        """Subscribe this session with Arlo system."""
        self._get_event_stream()
        self._subscribe_myself()

    @property
    def unsubscribe(self):
        """Unsubscribe this session."""
        self._unsubscribe_myself()
        self._close_event_stream()

    @mode.setter
    def mode(self, mode):
        """Set Arlo camera mode.

        :param mode: arm, disarm
        """
        if mode not in ACTION_MODES.keys():
            return "Invalid mode"
        self.__run_action(
            method='SET',
            resource='modes',
            mode=mode,
            publish_response=True)
        self.update()

    def update(self):
        """Update object properties."""
        self._attrs = self._session.refresh_attributes(self.name)

# vim:sw=4:ts=4:et:



# coding: utf-8
"""Base Python Class file for Netgear Arlo camera module."""
import logging
import requests

from pyarlo.base_station import ArloBaseStation
from pyarlo.camera import ArloCamera
from pyarlo.media import ArloMediaLibrary
from pyarlo.const import (
    BILLING_ENDPOINT, DEVICES_ENDPOINT,
    FRIENDS_ENDPOINT, LOGIN_ENDPOINT, PROFILE_ENDPOINT,
    PRELOAD_DAYS, RESET_ENDPOINT)

_LOGGER = logging.getLogger(__name__)


class PyArlo(object):
    """Base object for Netgar Arlo camera."""

    def __init__(self, username=None, password=None,
                 preload=True, days=PRELOAD_DAYS):
        """Create a PyArlo object.

        :param username: Arlo user email
        :param password: Arlo user password
        :param preload: Boolean to preload video library.
        :param days: If preload, number of days to lookup.

        :returns PyArlo base object
        """
        self.authenticated = None
        self.country_code = None
        self.date_created = None
        self.userid = None
        self.__token = None
        self.__headers = None
        self.__params = None

        # set username and password
        self.__password = password
        self.__username = username
        self.session = requests.Session()
        self.__base_stations = []

        # login user
        self.login()

        # pylint: disable=invalid-name
        self.ArloMediaLibrary = ArloMediaLibrary(self,
                                                 preload=preload,
                                                 days=days)

    def __repr__(self):
        """Object representation."""
        return "<{0}: {1}>".format(self.__class__.__name__, self.userid)

    def login(self):
        """Login to the Arlo account."""
        _LOGGER.debug("Creating Arlo session")
        self._authenticate()

    def _authenticate(self):
        """Authenticate user and generate token."""
        self.cleanup_headers()
        url = LOGIN_ENDPOINT
        data = self.query(url, method='POST')

        if isinstance(data, dict) and data.get('success'):
            data = data.get('data')
            self.authenticated = data.get('authenticated')
            self.country_code = data.get('countryCode')
            self.date_created = data.get('dateCreated')
            self.__token = data.get('token')
            self.userid = data.get('userId')

            # update header with the generated token
            self.__headers['Authorization'] = self.__token

    def cleanup_headers(self):
        """Reset the headers and params."""
        headers = {'Content-Type': 'application/json'}
        headers['Authorization'] = self.__token
        self.__headers = headers

        params = {'email': self.__username, 'password': self.__password}
        self.__params = params

    def query(self,
              url,
              method='GET',
              extra_params=None,
              extra_headers=None,
              retry=3,
              raw=False,
              stream=False):
        """
        Return a JSON object or raw session.

        :param url:  Arlo API URL
        :param method: Specify the method GET, POST or PUT. Default is GET.
        :param extra_params: Dictionary to be appended on request.body
        :param extra_headers: Dictionary to be apppended on request.headers
        :param retry: Attempts to retry a query. Default is 3.
        :param raw: Boolean if query() will return request object instead JSON.
        :param stream: Boolean if query() will return a stream object.
        """
        response = None
        loop = 0

        # always make sure the headers and params are clean
        self.cleanup_headers()

        while loop <= retry:

            # override request.body or request.headers dictionary
            if extra_params:
                params = self.__params
                params.update(extra_params)
            else:
                params = self.__params
            _LOGGER.debug("Params: %s", params)

            if extra_headers:
                headers = self.__headers
                headers.update(extra_headers)
            else:
                headers = self.__headers
            _LOGGER.debug("Headers: %s", headers)

            _LOGGER.debug("Querying %s on attempt: %s/%s", url, loop, retry)
            loop += 1

            # define connection method
            if method == 'GET':
                req = self.session.get(url, headers=headers, stream=stream)
            elif method == 'PUT':
                req = self.session.put(url, json=params, headers=headers)
            elif method == 'POST':
                req = self.session.post(url, json=params, headers=headers)

            req.raise_for_status()
            if req.status_code == 200:
                if raw:
                    _LOGGER.debug("Required raw object.")
                    response = req
                else:
                    response = req.json()

                # leave if everything worked fine
                break

        return response

    @property
    def cameras(self):
        """Return all cameras linked on Arlo account."""
        return self.devices['cameras']

    @property
    def base_stations(self):
        """Return all base stations linked on Arlo account."""
        return self.devices['base_station']

    @property
    def devices(self):
        """Return all devices on Arlo account."""
        devices = {}
        devices['cameras'] = []
        devices['base_station'] = []

        url = DEVICES_ENDPOINT
        data = self.query(url)

        for device in data.get('data'):
            name = device.get('deviceName')
            if ((device.get('deviceType') == 'camera' or
                 device.get('deviceType') == 'arloq' or
                 device.get('deviceType') == 'arloqs') and
                    device.get('state') == 'provisioned'):
                devices['cameras'].append(ArloCamera(name, device, self))

            if device.get('deviceType') == 'basestation' and \
               device.get('state') == 'provisioned':
                base = ArloBaseStation(name, device, self.__token, self)
                devices['base_station'].append(base)
                self.__base_stations.append(base)

        return devices

    def lookup_camera_by_id(self, device_id):
        """Return camera object by device_id."""
        camera = list(filter(
            lambda cam: cam.device_id == device_id, self.cameras))[0]
        if camera:
            return camera

    def refresh_attributes(self, name):
        """Refresh attributes from a given Arlo object."""
        url = DEVICES_ENDPOINT
        data = self.query(url).get('data')
        for device in data:
            if device.get('deviceName') == name:
                return device

    @property
    def unseen_videos_reset(self):
        """Reset the unseen videos counter for all cameras."""
        return self.query(RESET_ENDPOINT).get('success')

    @property
    def billing_information(self):
        """Return billing json."""
        url = BILLING_ENDPOINT
        return self.query(url)

    @property
    def shared_users(self):
        """Return shared users json."""
        url = FRIENDS_ENDPOINT
        return self.query(url)

    @property
    def profile(self):
        """Return user profile json."""
        url = PROFILE_ENDPOINT
        return self.query(url)

    @property
    def is_connected(self):
        """Connection status of client with Arlo system."""
        return bool(self.authenticated)

    def update(self):
        """Refresh object."""
        self._authenticate()

# vim:sw=4:ts=4:et:



# coding: utf-8
"""Generic Python Class file for Netgear Arlo camera module."""
import logging
from pyarlo.const import (
    RESET_CAM_ENDPOINT, STREAM_ENDPOINT, STREAMING_BODY)
from pyarlo.media import ArloMediaLibrary
from pyarlo.utils import http_get

_LOGGER = logging.getLogger(__name__)


class ArloCamera(object):
    """Arlo Camera module implementation."""

    def __init__(self, name, attrs, arlo_session):
        """Initialize Arlo camera object.

        :param name: Camera name
        :param attrs: Camera attributes
        :param arlo_session: PyArlo shared session
        """
        self.name = name
        self._attrs = attrs
        self._session = arlo_session

    def __repr__(self):
        """Representation string of object."""
        return "<{0}: {1}>".format(self.__class__.__name__, self.name)

    # pylint: disable=invalid-name
    @property
    def device_id(self):
        """Return device_id."""
        return self._attrs.get('deviceId')

    @property
    def device_type(self):
        """Return device_type."""
        return self._attrs.get('deviceType')

    @property
    def model_id(self):
        """Return model_id."""
        return self._attrs.get('modelId')

    @property
    def hw_version(self):
        """Return hardware version."""
        return self._attrs.get('properties').get('hwVersion')

    @property
    def timezone(self):
        """Return timezone."""
        return self._attrs.get('properties').get('olsonTimeZone')

    @property
    def unique_id(self):
        """Return unique_id."""
        return self._attrs.get('uniqueId')

    @property
    def user_id(self):
        """Return userID."""
        return self._attrs.get('userId')

    @property
    def unseen_videos(self):
        """Return number of unseen videos."""
        self.update()
        return self._attrs.get('mediaObjectCount')

    @property
    def unseen_videos_reset(self):
        """Reset the unseen videos counter."""
        url = RESET_CAM_ENDPOINT.format(self.unique_id)
        ret = self._session.query(url).get('success')
        self.update()
        return ret

    @property
    def user_role(self):
        """Return userRole."""
        return self._attrs.get('userRole')

    @property
    def last_image(self):
        """Return last image capture by camera."""
        self.update()
        return http_get(self._attrs.get('presignedLastImageUrl'))

    @property
    def last_video(self):
        """Return the last <ArloVideo> object from camera."""
        library = ArloMediaLibrary(self._session, preload=False)
        try:
            return library.load(only_cameras=[self], limit=1)[0]
        except IndexError:
            return None

    def videos(self, days=180):
        """
        Return all <ArloVideo> objects from camera given days range

        :param days: number of days to retrieve
        """
        library = ArloMediaLibrary(self._session, preload=False)
        try:
            return library.load(only_cameras=[self], days=days)
        except (AttributeError, IndexError):
            return []

    @property
    def captured_today(self):
        """Return list of <ArloVideo> object captured today."""
        return self.videos(days=0)

    @property
    def play_last_video(self):
        """Play last <ArloVideo> recorded from camera."""
        video = self.last_video
        return video.download_video()

    @property
    def xcloud_id(self):
        """Return X-Cloud-ID attribute."""
        return self._attrs.get('xCloudId')

    @property
    def get_battery_level(self):
        """Get the camera battery level."""
        base = self._session.base_stations[0]
        return base.get_camera_battery_level[self.device_id]

    @property
    def get_signal_strength(self):
        """Get the camera Signal strength."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                return cam['signalStrength']

        return None

    @property
    def get_brightness(self):
        """Get the brightness property of camera."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                return cam['brightness']

        return None

    @property
    def get_mirror_state(self):
        """Get the brightness property of camera."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                return cam['flip']

        return None

    @property
    def get_flip_state(self):
        """Get the brightness property of camera."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                return cam['mirror']

        return None

    @property
    def get_powersave_mode(self):
        """Get the brightness property of camera."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                return cam['powerSaveMode']

        return None

    @property
    def is_camera_connected(self):
        """Connectivity status of Cam with Base Station."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                return bool(cam['connectionState'] == 'available')

        return None

    @property
    def get_motion_detection_sensitivity(self):
        """Sensitivity level of Camera motion detection."""
        base = self._session.base_stations[0]
        props = base.get_camera_properties
        if not props:
            return None
        for cam in props:
            if cam['serialNumber'] == self.device_id:
                this_cam = cam
        triggers = this_cam['capabilities'][9]['Triggers']
        return triggers[0]['sensitivity']['default']

    def live_streaming(self):
        """Return live streaming generator."""
        url = STREAM_ENDPOINT

        # override params
        params = STREAMING_BODY
        params['from'] = "{0}_web".format(self.user_id)
        params['to'] = self.device_id
        params['resource'] = "cameras/{0}".format(self.device_id)
        params['transId'] = "web!{0}".format(self.xcloud_id)

        # override headers
        headers = {'xCloudId': self.xcloud_id}

        _LOGGER.debug("Streaming device %s", self.name)
        _LOGGER.debug("Device params %s", params)
        _LOGGER.debug("Device headers %s", headers)

        ret = self._session.query(url,
                                  method='POST',
                                  extra_params=params,
                                  extra_headers=headers)

        _LOGGER.debug("Streaming results %s", ret)
        if ret.get('success'):
            return ret.get('data').get('url')
        return ret.get('data')

    def update(self):
        """Update object properties."""
        self._attrs = self._session.refresh_attributes(self.name)

# vim:sw=4:ts=4:et:



# coding=utf-8
"""Python Arlo setup script."""
from setuptools import setup

setup(
    name='pyarlo',
    packages=['pyarlo'],
    version='0.0.5',
    description='Python Arlo is a library written in Python 2.7/3x ' +
                'that exposes the Netgear Arlo cameras as Python objects.',
    author='Marcelo Moreira de Mello',
    author_email='tchello.mello@gmail.com',
    url='https://github.com/tchellomello/python-arlo',
    license='LGPLv3+',
    include_package_data=True,
    install_requires=['requests', 'sseclient-py'],
    test_suite='tests',
    keywords=[
        'arlo',
        'netgear',
        'camera',
        'home automation',
        'python',
        ],
    classifiers=[
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ' +
        'GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules'
        ],
)
